%  RunCulture.m -> DefineMedium()
%
%         #coculture: data structure for initialization
%         #bio_t0: array of initial biomass for each model
%         #time_steps: total FBA time points
%         #ctns_medium: array of initial metabolite level in the medium
%

function [coculture] = DefineMedium(coculture, bio_t0, time_steps, ctns_medium, mets_medium );

    % Initialize concentrations in the medium
    coculture.bio  = zeros( length(coculture.species) , length(time_steps)  ) ;
    coculture.ctns = zeros( length(coculture.mets   ) , length(time_steps)  ) ;

    % Initial biomass and initial concentrations
    coculture.bio(:,1) = bio_t0;

    for i=1:length(mets_medium)
        id = strmatch( mets_medium{i} , coculture.mets , 'exact');
        if ~isempty(id)

			% Normal initialization
			coculture.ctns( id ,1)  = ctns_medium(i);

            if(strcmp(mets_medium{i}, '(NH4)2SO4(e)[env]') == 1)
                %(1)Reset Ammonium Sulfate[env]
                coculture.ctns( id ,1)  = 0;
                %(2)Instead, initialize Ammonium[env] and Sulfate[env]
                id1 = strmatch( 'nh4[env]' , coculture.mets , 'exact');
                coculture.ctns( id1 ,1)  = ctns_medium(i);

                id2 = strmatch( 'so4[env]' , coculture.mets , 'exact');
                coculture.ctns( id2 ,1)  = ctns_medium(i);
            end

        else % alias name e.g. ala_L[e] in Mba instead of ala-L[e] in environment definition
            a = strmatch( mets_medium{i} , coculture.mets_alias , 'exact');

            if(isempty(a) )
               disp(['============================================='])
               disp(['  No species can uptake '  char(mets_medium{i})  ' '  char(mets_medium{i})  '.'])
               a;
            else
            id_hash = coculture.mets_alias_hashid(a) ;
            coculture.ctns( id_hash ,1)  = ctns_medium(i);
            end
        end
    end

	coculture.reservoir =  coculture.ctns(:,1);
	deltaX = coculture.ctns(:,1);
	Dilution = coculture.tankflow / coculture.volume;
	coculture.reservoir =  deltaX / Dilution;

end
