% This function is the core of our ecosystem modeling framework
%
% Input: models, parameters, medium
%
% Output: 
%         (1)coculture: all dynamics of the ecosystems
%                       (growth, biomass, flux, concentrations)
%
function [cocultureout] = RunCulture(media, coculture, models, params)


    % --- NOT USED, but leave it for reference ---
        k = length(coculture.species);
        %No death
        for k=1:length(coculture.species)
            models(k).death_rate = 0; % fixed death rate
        end

        %We don't release metabolties from dead cell in our EBC paper
        degradation = 0.2*ones(1,k) ;  % 20 percentage of biomass can be released upon death
        %Ideally death_release should be the coeff in Vgro,
        death_release = 0.3*ones(1,k); % 0.3mmol metabolite released per gram of biomass
    % --- NOT USED, but leave it for reference ---


    coculture.volume = params.volume;

    coculture.tankflow = params.dilution * coculture.volume;
		%Dilution = Flow/Volume [hr-1] should be exactly the same as growth


    if (isfield(params,'init_bio_abs')) %use absolute concentration parameter if present
       Initial_BIO_ratio = params.init_bio_abs;
       Initial_BIO = Initial_BIO_ratio;
    elseif (isfield(params,'init_bio_relative')) %else use relative concentration parameter and scale to biomass_order
        Initial_BIO_ratio = params.init_bio_relative;
        Initial_BIO = Initial_BIO_ratio / sum(Initial_BIO_ratio) * params.biomass_order;
    else %default start all at same concentration, sum to biomass_order
        Initial_BIO_ratio = ones(1,length(models));
        Initial_BIO = Initial_BIO_ratio / sum(Initial_BIO_ratio) * params.biomass_order;
    end

    %Calculate the initial ctns per unit of transfer (e.g. every 6 hrs)
    if params.cultivation == 0        %batch
        %media.concentrations = media.concentrations*params.totaltime;
    elseif params.cultivation == 2
        error('Serial transfer not implemented yet')
    elseif params.cultivation == 3    %fed-batch
        media.concentrations = media.concentrations*coculture.transfer_hr;
        error('Fed batch not implemented at the moment')
    end


	% This has to be done after  scaling with coculture.tankflow
	%concentration_base = 1;
	clear mets_medium; clear ctns_medium;
	mets_medium = media(1).mets;
    mets_medium = strrep(mets_medium, '[e]', '[env]');
    mets_medium = strrep(mets_medium, '(e)', '[env]');

    initial_concentration = params.init_media_concentration; % mM

    if isfield(media,'concentrations')
        ctns_medium = media(1).concentrations; % * concentration_base;
    else
        ctns_medium = initial_concentration * ones(size(mets_medium));
    end

	MM.Vmax(1:length(coculture.mets) ) = params.Vmax;
	MM.km(1:length(coculture.mets)) =    params.km;

    coculture.totaltime = params.totaltime;
    coculture.delta_t = params.delta_t;
    coculture.Vmax0 = min(MM.Vmax);
    coculture.km0   = min(MM.km);

    coculture.dietswitchtime = Inf;

	time_steps = [0:params.delta_t:params.totaltime];

	coculture.mediumname = strrep(params.media_csv,'.csv','');

	%[2]run FBA---------------------------------------------------------
	% ------------------Starting the growth phase over time--------------------
	%make a backup and for reinitialization at each time t
	init_coculture = coculture;
	init_models = models;

	% RESET for each different ratio start
	clear coculture; coculture = init_coculture;
	clear models; models = init_models;

    % do deletions
    if isfield(params,'deleterxn')
        deletestrings = {};
        if iscell(params.deleterxn)
            deletestrings = params.deleterxn;
        else
            deletestrings{1} = params.deleterxn;
        end

        for i = 1:length(deletestrings)
            DComponents = textscan(deletestrings{i},'s%dr%d');
            models(DComponents{1}).ub(DComponents{2}) = 1e-10;
            models(DComponents{1}).lb(DComponents{2}) = -1e-10;
        end
    end

    if isfield(params,'deletegene')
        deletestrings = {};
        if iscell(params.deletegene) 
            deletestrings = params.deletegene;
        else
            deletestrings{1} = params.deletegene;
        end

        deletedgenes = cell(10,1);
        for i = 1:length(deletestrings)
            DComponents = textscan(deletestrings{i},'s%dg%d');
            deletedgenes{DComponents{1}} = [deletedgenes{DComponents{1}} DComponents{2}];
        end
        for i = 1:length(models)
            if (length(deletedgenes{i}) > 1)
                ToBeD = CalcRxnLoss(models(i), deletedgenes{i});
                models(i).ub(ToBeD) = 1e-10;
                models(i).lb(ToBeD) = -1e-10;
            end
        end
    end


	%Make a copy of non-initialized structure
	coculture0 = coculture;
	% % Initialize ctns(t0) and then GROW the cell: mets_medium is defined in the UserChanges.m
	coculture = DefineMedium(coculture0, Initial_BIO, time_steps  , ctns_medium , mets_medium);

	if isfield(params, 'starting_condition_file')
		coculture.bio(:,1) = params.starting_condition.bio(:,1);
		coculture.ctns(:,1) = params.starting_condition.ctns(:,1);
	end

	[cocultureout, ~]= GrowCell(params, degradation , death_release, models, coculture, coculture.totaltime, coculture.delta_t, time_steps, MM);

end

