function [v_sol_minAV_flux ,obj_sol_minAV,status_min_AV,extra] = min_Flux_AV(Obj, FBAgrowth, Rxns, Mets, S , Bvector , Lbound , Ubound , CTYPE, VARTYPE, verbose);
% This script fix the FBAgrowth and try to minimize flux activity

	%%%RUN SECOND OPTIMIZATION TO SELECT SOLUTION WHICH MINIMIZES SUM OF ABSOLUTE VALUES OF THE FLUXES%%%
	%ADD THE CONSTRAINT THAT THE MAXIMIZED VALUES FROM PREVIOUS OPTIMIZATION ARE NOW FIXED
	obj_fluxes = find(Obj ~= 0);
	new_rows = zeros( length(obj_fluxes) , size(S,2));
	constr_rxns = sub2ind( [length(obj_fluxes) size(S,2)] , (1:length(obj_fluxes)) , obj_fluxes' );
	new_rows(constr_rxns) = 1;

	S = [S ; new_rows];
	Bvector = [Bvector ; FBAgrowth];


	%ADJUST S MATRIX TO ADD A NEW VARIABLE CORRESPODING TO EACH FLUX, WHICH WILL FUNCTION AS A DISTANCE FROM ZERO
	new_variables = zeros( size(S,1) , size(S,2) );
	S = [S new_variables];


	%NUMBER OF METABOLITES PLUS ADDITIONAL CONSTRAINTS(ROWS OF S, CONSTRAINTS)
	M = size(S,1);

	%NUMBER OF REACTIONS AND ADDITION VARIABLES(COLUMNS OF S, VARIABLES)
	N = size(S,2);


	%ADD TWO NEW CONSTRAINTS FOR EACH FLUX TO ACCOUNT FOR THE CASES OF THE FLUX BEING LESS THAN OR GREATER THAN ZERO
	new_constraints = zeros( size(S,2) , size(S,2) );

	upper_const = 1:2:size(new_constraints,1);
	lower_const = 2:2:size(new_constraints,1);

	fluxes_to_const = 1:(N/2);
	const_vars = fluxes_to_const + (N/2);

	upper_const_flux_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , upper_const , fluxes_to_const);
	upper_const_var_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , upper_const , const_vars);
	new_constraints(upper_const_flux_i) = 1;
	new_constraints(upper_const_var_i) = -1;

	lower_const_flux_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , lower_const , fluxes_to_const);
	lower_const_var_i = sub2ind([size(new_constraints,1) size(new_constraints,2)] , lower_const , const_vars);
	new_constraints(lower_const_flux_i) = -1;
	new_constraints(lower_const_var_i) = -1;

	Bvector = [Bvector ; zeros(size(new_constraints,1),1)];
	S = [S ; new_constraints];


	%SET NEW OBJECTIVE WHICH MINIMIZES THE SUM OF THE NEW VARIABLES REPRESENTING EACH OF THE FLUXES DISTANCE FROM ZERO
	OBJ_min_flux = [zeros( length(Obj) , 1) ; ones( length(Obj) , 1 )];
	Lbound = [Lbound ; zeros( (N/2) , 1)];
	Ubound = [Ubound ; Inf( (N/2) , 1)];


	%MINIMIZE SUM OF ABSOLUTE VALUES OF FLUXES
	min_max = 1;


	%C-TYPE FOR NEW CONSTRAINTS HAS UPPER BOUND
	CTYPE = [ char( 83*ones(1,M) )' ; char( 85*ones( 1,size(new_constraints,1)) )'];


	%VARTYPES STILL ALL CONTINUOUS
	VARTYPE = char( 67*ones(1,size(new_constraints,2)) )';


	param.msglev = 1;
	param.scale = 2;  % default is 1  original(2)
	param.itlim = 10000;
	param.LPsolver = 1; % for glpk
	param.save_pb = 0;  % for glpk

	%***** Turn off presolver may keep away from rounding error, but not necessarily will *****
	param.presol = 1;


	%PERFORM OPTIMIZATION
	[v_WT_msf ,obj_sol_minAV,status_min_AV,extra] = glpk(OBJ_min_flux , S , Bvector , Lbound , Ubound , CTYPE, VARTYPE , min_max, param);


	x = find((v_WT_msf - Lbound) < -1e-6);

	if (status_min_AV ~= 5 || ~isempty(x))
		%Try again with glpkbug trick
		param.presol = 0;
		param.itlim = -1;
		param.tmlim = -1;
		indices_eq = (find((Ubound-Lbound)==0));
		Ubound(indices_eq) = Ubound(indices_eq) + 1e-7;

		[v_WT_msf ,obj_sol_minAV,status_min_AV,extra] = glpk(OBJ_min_flux , S , Bvector , Lbound , Ubound , CTYPE, VARTYPE , min_max, param);
		x = find((v_WT_msf - Lbound) < -1e-6);
		disp('Turn off presolvor and Add 1e-7 error tolerence');

		if (status_min_AV ~= 5 || ~isempty(x))
			%In rare cases, call relaxed method
			m= length(Mets); n = length(Rxns);
			[v_sol_minAV_flux_relax ,obj_sol_minAV_relax,status_min_AV_relax,extra] = min_Flux_AV_relaxed(Obj, FBAgrowth, Rxns, Mets, S(1:m, 1:n) , Bvector(1:m) , Lbound(1:n) , Ubound(1:n) , CTYPE(1:m), VARTYPE(1:n));
			x = find((v_sol_minAV_flux_relax - Lbound(1:n)) < -1e-6);

			if (status_min_AV_relax ~= 5 || ~isempty(x))

				status_min_AV
				FBAgrowth

				disp(['Still fail to get Min_Total_Flux of FBAgrowth=' num2str(FBAgrowth)]) ;
				disp(' even if relaxed minAV is called!!! ')
				disp(' This case might need larger numerical error bar ')

				v_WT_msf = 0*v_WT_msf;
				obj_sol_minAV = 0;
				status_min_AV = 4;

			end
		end
	end

	v_sol_minAV_flux = v_WT_msf( 1:length(Rxns) );

if (verbose)
    disp(['                 Min_Total_Flux status = ' num2str(status_min_AV) '   activity =' num2str(sum(abs(v_sol_minAV_flux)))])
end
