% run_experiments.m
% top level for running FBA coculture model
%
% Framework by Hsuan-Chao Chiu, revised by Colin McNally

function run_experiments(configfile, job_idd)

	params = ReadConfig(configfile);
	job_id = str2num(job_idd);
	disp(['  job_id = ' num2str(job_id)])


	% Create output folder
	% MKDIR output folder if does not exist

    output_path = fullfile(params.output_dir);
    if ~exist(output_path,'dir')
        eval(['mkdir ' output_path]);  % Create project folder
    end

	media = ReadMediumMetsConcen(fullfile(params.input_dir,params.csv_dir,params.media_csv));
    [coculture , models]= CreateModel( fullfile(params.input_dir,params.model_dir), params.dictionary, params.species, params);

	if isfield(params, 'starting_condition_file')
		SCconds = load(params.starting_condition_file);
		SCconds = SCconds.cocultures(1);
		SCconds = rmfield(SCconds, {'mu_model','death_rate','mets','metNames'});
		SCconds.bio = SCconds.bio(:,max(SCconds.finaltimestep));
		SCconds.ctns = SCconds.ctns(:,max(SCconds.finaltimestep));
		params.starting_condition = SCconds;
	end

    if isfield(params, 'tiling_variable')
        for i = 1:length(eval(['params.' params.tiling_variable{1}]))
            subparams(i) = params;
            eval(['subparams(' num2str(i) ').' params.tiling_variable{1} ' = params.' params.tiling_variable{1} '{' num2str(i) '};']);
        end
    else
        subparams(1) = params;
    end

    detailfields = {'allflux','absfluxactivity','glpkstatus','lb','ub','glpkflux','v_model2medium'};

    for i = 1:length(subparams)

        rescoculture = RunCulture(media, coculture, models, subparams(i));

        if params.output_details
            cocultures(i) = orderfields(rescoculture);
        else
            cocultures(i) = orderfields(rmfield(rescoculture, detailfields));
        end
    end

    % Save the results %
    if ~exist(params.output_dir, 'dir')
		mkdir(params.output_dir);
	end
	str_mat = fullfile(params.output_dir, [params.output_filename '.mat']);

    if isfield(params, 'tiling_variable')
       for i = 1:length(subparams)
            tileparam{i} = eval(['subparams(' num2str(i) ').' params.tiling_variable{1}]);
       end
    else
        tileparam = '';
    end

	%Move detailed information into another variable
    if params.output_details
        cocultures_detail=[];
        for i=1:length(detailfields)
            for j = 1:length(subparams)
                eval(['cocultures_detail(' num2str(j) ').' detailfields{i} ' = cocultures(' num2str(j) ').' detailfields{i} ';']);

            end
        end
        cocultures = rmfield(cocultures,detailfields);
    end
	
    if (params.output_details)
        save(str_mat,'cocultures','cocultures_detail','tileparam');
    else
        save(str_mat,'cocultures','tileparam');
    end
end
